import os
import numpy as np
import pandas as pd
from rpy2.robjects.packages import importr
from rpy2.robjects import FloatVector, numpy2ri
from datetime import datetime


base = importr('base')
utils = importr('utils')
pth = os.path.abspath(os.getcwd())

stats = importr('stats')
graphics = importr('graphics')
grdevices = importr('grDevices')


try:
    base.suppressWarnings(utils.install_packages("forecast"))
except:
    pass

try:
    ahead = importr('ahead')
except:
    base.suppressWarnings(utils.install_packages("ahead",
                                                 repos=pth + "/r-ahead-forge",
                                                 type="source"))
    ahead = importr('ahead')


class DynamicRegressor():

    def __init__(self, h=5, level=95, 
                 weights=[1/3, 1/3, 1/3], 
                 pi="E", date_formatting="original"):
        
        self.h = h
        self.level = level
        self.weights = weights
        self.pi = pi
        self.date_formatting=date_formatting

        self.fcast = None
        self.averages = None
        self.ranges = None        


    def forecast(self, df):            
        
        # obtain dates 'forecast' -----

        input_dates = df['date'].values 
        n_input_dates = len(input_dates)        
        input_dates_ms = [datetime.strptime(str(input_dates[i]), "%Y-%m-%d").timestamp()*1000 for i in range(n_input_dates)]    

        frequency = pd.infer_freq(pd.DatetimeIndex(input_dates))
        output_dates = np.delete(pd.date_range(start=input_dates[-1], 
            periods=self.h+1, freq=frequency).values, 0).tolist()  
        
        df_dates = pd.DataFrame({'date': output_dates})
        output_dates = pd.to_datetime(df_dates['date']).dt.date                                

        # obtain time series forecast -----

        input_series = df.drop(columns=['date']).values
        y = stats.ts(FloatVector([item for sublist in input_series.tolist() for item in sublist]))
        self.fcast = ahead.dynrmf(y, h=self.h, level=self.level, pi=self.pi)        


        # result -----

        if (self.date_formatting == "original"): 
            fcast_dates = [datetime.strftime(output_dates[i], "%Y-%m-%d") for i in range(self.h)]            
            self.averages = [[fcast_dates[i], self.fcast.rx2['mean'][i]] for i in range(self.h)]
            self.ranges = [[fcast_dates[i], self.fcast.rx2['lower'][i], self.fcast.rx2['upper'][i]] for i in range(self.h)]            
            return self         
        
        if (self.date_formatting == "ms"):  
            fcast_dates_ms = [int(datetime.strptime(str(output_dates[i]), "%Y-%m-%d").timestamp()*1000) for i in range(self.h)]
            self.averages = [[fcast_dates_ms[i], self.fcast.rx2['mean'][i]] for i in range(self.h)]
            self.ranges = [[fcast_dates_ms[i], self.fcast.rx2['lower'][i], self.fcast.rx2['upper'][i]] for i in range(self.h)]        
            return self         




import os
import numpy as np
import pandas as pd
from rpy2.robjects.packages import importr
from rpy2.robjects import FloatVector, numpy2ri
from datetime import datetime


base = importr('base')
utils = importr('utils')
pth = os.path.abspath(os.getcwd())

stats = importr('stats')
graphics = importr('graphics')
grdevices = importr('grDevices')


try:
    base.suppressWarnings(utils.install_packages("forecast"))
except:
    pass

try:
    ahead = importr('ahead')
except:
    base.suppressWarnings(utils.install_packages("ahead",
                                                 repos=pth + "/r-ahead-forge",
                                                 type="source"))
    ahead = importr('ahead')


class Ridge2Regressor():

    def __init__(self, h=5, level=95, 
                 lags=1, nb_hidden=5,
                 nodes_sim="sobol", activation="relu",
                 a=0.01, lambda_1=0.1, lambda_2=0.1, 
                 date_formatting="original", seed=123):
        
        self.h = h
        self.level = level
        self.lags = lags
        self.nb_hidden = nb_hidden
        self.nodes_sim = nodes_sim
        self.activation = activation
        self.a = a
        self.lambda_1 = lambda_1
        self.lambda_2 = lambda_2
        self.date_formatting=date_formatting
        self.seed = seed
        
        self.averages = None
        self.ranges = None   


    def forecast(self, df): 

        assert (len(df.columns) >= 3), "Must have number of columns >= 3" 

        n_series = len(df.columns) - 1
        averages = []
        ranges = []

        # obtain dates 'forecast' -----

        # to be put in utils/ as a function (DRY)

        input_dates = df['date'].values 
        n_input_dates = len(input_dates)        
        input_dates_ms = [datetime.strptime(str(input_dates[i]), "%Y-%m-%d").timestamp()*1000 for i in range(n_input_dates)]    

        frequency = pd.infer_freq(pd.DatetimeIndex(input_dates))
        output_dates = np.delete(pd.date_range(start=input_dates[-1], 
            periods=self.h+1, freq=frequency).values, 0).tolist()  
        
        df_dates = pd.DataFrame({'date': output_dates})
        output_dates = pd.to_datetime(df_dates['date']).dt.date 


        # obtain time series forecast -----

        # to be put in utils/ as a function (DRY)

        input_series = df.drop(columns=['date'])        
        input_series_tolist = input_series.values.tolist()
        xx = [item for sublist in input_series_tolist for item in sublist]  
        m = stats.ts(base.matrix(FloatVector(xx), byrow=True, 
                                 nrow=len(input_series_tolist)))      

        self.fcast = ahead.ridge2f(m, h=self.h, level=self.level, 
                               lags=self.lags, nb_hidden=self.nb_hidden, 
                               nodes_sim=self.nodes_sim, activ=self.activation, 
                               a = self.a, lambda_1=self.lambda_1, 
                               lambda_2 = self.lambda_2, seed=self.seed)

        # result -----
        
        if self.date_formatting == "original": 
            # to be put in utils/ as a function (DRY)
            for j in range(n_series):
                averages_series_j  = []
                ranges_series_j  = []
                for i in range(self.h): 
                    date_i = datetime.strftime(output_dates[i], "%Y-%m-%d")    
                    index_i_j = i+j*self.h       
                    averages_series_j.append([date_i, 
                        self.fcast.rx2['mean'][index_i_j]])
                    ranges_series_j.append([date_i, 
                        self.fcast.rx2['lower'][index_i_j], self.fcast.rx2['upper'][index_i_j]])
                averages.append(averages_series_j)
                ranges.append(ranges_series_j) 


        if self.date_formatting == "ms": 
            # to be put in utils/ as a function (DRY)
            for j in range(n_series):
                averages_series_j  = []
                ranges_series_j  = []
                for i in range(self.h): 
                    date_i = int(datetime.strptime(str(output_dates[i]), "%Y-%m-%d").timestamp()*1000)
                    index_i_j = i+j*self.h            
                    averages_series_j.append([date_i, 
                        self.fcast.rx2['mean'][index_i_j]])
                    ranges_series_j.append([date_i, 
                        self.fcast.rx2['lower'][index_i_j], self.fcast.rx2['upper'][index_i_j]])
                averages.append(averages_series_j)
                ranges.append(ranges_series_j) 


        self.averages = averages
        self.ranges = ranges                         

        return self


"""Top-level package for ahead."""

__author__ = """T. Moudiki"""
__email__ = 'thierry.moudiki@gmail.com'
__version__ = '0.1.1'

from .DynamicRegressor import DynamicRegressor
from .EAT import EAT
from .Ridge2 import Ridge2Regressor
from .VAR import VAR


__all__ = ["DynamicRegressor", "EAT", "Ridge2Regressor", "VAR"]



#!/usr/bin/env python

"""Tests for `ahead` package."""

# python -m unittest

import unittest
import pandas as pd
from ahead import EAT, DynamicRegressor, Ridge2Regressor, VAR


# Univariate dataset 
# Data frame containing the time series 
dataset_uni = {
    'date' : ['2020-01-01', '2020-02-01', '2020-03-01', '2020-04-01', '2020-05-01'],
    'value' : [34, 30, 35.6, 33.3, 38.1],    
}
df_uni = pd.DataFrame(dataset_uni)

# Multivariate dataset 
# Data frame containing the time series 
dataset_multi = {
 'date' : ['2001-01-01', '2002-01-01', '2003-01-01', '2004-01-01', '2005-01-01'],
 'series1' : [34, 30, 35.6, 33.3, 38.1],    
 'series2' : [4, 5.5, 5.6, 6.3, 5.1],
 'series3' : [100, 100.5, 100.6, 100.2, 100.1]}
df_multi = pd.DataFrame(dataset_multi)


# Forecasting horizon
h = 5


e1 = EAT(h = h, weights = [0.5, 0.4, 0.1], date_formatting = "ms")
e2 = EAT(h = h, pi="T", date_formatting = "original")        
e1.forecast(df_uni)
e2.forecast(df_uni)

e3 = DynamicRegressor(h = h, date_formatting = "ms")
e4 = DynamicRegressor(h = h, date_formatting = "original")
e3.forecast(df_uni)
e4.forecast(df_uni)

e5 = Ridge2Regressor(h = h, date_formatting = "ms")
e6 = Ridge2Regressor(h = h, date_formatting = "original")
e5.forecast(df_multi)
e6.forecast(df_multi)
print("Ridge2Regressor ---------- \n")
print(e5.averages)
print("\n")
print(e5.ranges)
print("\n")
print(e6.averages)
print("\n")
print(e6.ranges)
print("\n")

e7 = VAR(h = h, date_formatting = "ms")
e8 = VAR(h = h, date_formatting = "original")
e7.forecast(df_multi)
e8.forecast(df_multi)
print("VAR ---------- \n")
print(e7.averages)
print("\n")
print(e7.ranges)
print("\n")
print(e8.averages)
print("\n")
print(e8.ranges)
print("\n")


class TestAhead(unittest.TestCase):
    """Tests for `ahead` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_eat(self):            
        self.assertAlmostEqual(e1.averages[0][1], 34.28750002378786)
        self.assertAlmostEqual(e1.ranges[0][1], 28.95860083805203)
        self.assertAlmostEqual(e2.averages[0][1], 37.833333650504755)
        self.assertAlmostEqual(e2.ranges[0][1], 35.39276803598083)
        self.assertEqual(e1.averages[0][0], 1590962400000)
        self.assertEqual(e2.averages[0][0], '2020-06-01')

    def test_dynrm(self):        
        self.assertAlmostEqual(e3.averages[0][1], 36.79847159323566)
        self.assertAlmostEqual(e3.ranges[0][1], 29.66346091111926)
        self.assertAlmostEqual(e4.averages[0][1], 36.79847159323566)
        self.assertAlmostEqual(e4.ranges[0][1], 29.66346091111926)
        self.assertEqual(e3.averages[0][0], 1590962400000)        
        self.assertEqual(e4.averages[0][0], '2020-06-01')        
    
    def test_ridge2(self):
        self.assertAlmostEqual(e5.averages[0][0][1], 33.99538584327151)
        self.assertAlmostEqual(e6.averages[0][0][1], 33.99538584327151)
        self.assertAlmostEqual(e5.averages[0][0][0], 1136070000000)
        self.assertAlmostEqual(e6.averages[0][0][0], '2006-01-01')

    def test_var(self):            
        self.assertAlmostEqual(e7.averages[0][0][1], 31.998824593128404)
        self.assertAlmostEqual(e8.averages[0][0][1], 31.998824593128404)
        self.assertAlmostEqual(e7.averages[0][0][0], 1136070000000)
        self.assertAlmostEqual(e8.averages[0][0][0], '2006-01-01')




from setuptools import setup, find_packages
from codecs import open
from os import path

__version__ = "0.1.1"

here = path.abspath(path.dirname(__file__))


# get the dependencies and installs
with open(
    path.join(here, "requirements.txt"), encoding="utf-8"
) as f:
    all_reqs = f.read().split("\n")

install_requires = [
    x.strip() for x in all_reqs if "git+" not in x
]
dependency_links = [
    x.strip().replace("git+", "")
    for x in all_reqs
    if x.startswith("git+")
]

setup(
    name="ahead",
    version=__version__,
    description="Time series forecasting",
    long_description="A package for time series forecasting",
    #url="https://github.com/thierrymoudiki/GPopt",
    #download_url="https://github.com/thierrymoudiki/GPopt/tarball/"
    #+ __version__,
    license="BSD3 Clause Clear",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
    ],
    keywords="",
    packages=find_packages(exclude=["docs", "tests*"]),
    include_package_data=True,
    author="Thierry Moudiki",
    install_requires=install_requires,
    dependency_links=dependency_links,
    author_email="thierry.moudiki@gmail.com",
)
